package com.example.greenwatch;

import androidx.appcompat.app.AppCompatActivity;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.view.SurfaceView;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.example.greenwatch.adapters.AlarmHistoryListAdapter;
import com.example.greenwatch.adapters.DeviceListAdapter;
import com.example.greenwatch.models.Device;
import com.example.greenwatch.viewmodels.AudiodetectionAndAccelerometerViewModel;

import java.util.List;

public class AudiodetectionAndAccelerometerActivity extends AppCompatActivity implements SensorEventListener {
    private SensorManager accelerometerManager;
    private Sensor accelerometerSensor;
    private SurfaceView surfaceView;
    private TextView audiodetectionAndAccelerometerStatusMessage;
    private TextView tvAudiodetectionAndAccelerometerDeviceListRecyclerView;
    private TextView tvAudiodetectionAndAccelerometerAlarmHistoryListRecyclerView;
    private AudiodetectionAndAccelerometerViewModel mAudiodetectionAndAccelerometerViewModel;
    private Button backToMainActivity;

    private Permission permission = new Permission();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_audiodetection_and_accelerometer);

        surfaceView = (SurfaceView) findViewById(R.id.surfaceViewAudiodetectionAndAccelerometer);
        audiodetectionAndAccelerometerStatusMessage = (TextView) findViewById(R.id.tvAudiodetectionAndAccelerometerStatusmessage);
        tvAudiodetectionAndAccelerometerDeviceListRecyclerView = (TextView) findViewById(R.id.tvAudiodetectionAndAccelerometerDeviceListRecyclerView);
        tvAudiodetectionAndAccelerometerAlarmHistoryListRecyclerView = (TextView) findViewById(R.id.tvAudiodetectionAndAccelerometerAlarmHistoryListRecyclerView);
        backToMainActivity = (Button) findViewById(R.id.audiodetectionAndAccelerometerBackToMainActivity);

        RecyclerView recyclerView = findViewById(R.id.deviceListRecyclerView);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        recyclerView.setHasFixedSize(true);

        final DeviceListAdapter deviceListAdapter = new DeviceListAdapter();
        recyclerView.setAdapter(deviceListAdapter);

        RecyclerView alarmHistoryListRecyclerView = findViewById(R.id.alarmHistoryListRecyclerView);
        alarmHistoryListRecyclerView.setLayoutManager(new LinearLayoutManager(this));
        alarmHistoryListRecyclerView.setHasFixedSize(true);

        final AlarmHistoryListAdapter alarmHistoryListAdapter  = new AlarmHistoryListAdapter();
        alarmHistoryListRecyclerView.setAdapter(alarmHistoryListAdapter);

        backToMainActivity.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        mAudiodetectionAndAccelerometerViewModel = new ViewModelProvider(this).get(AudiodetectionAndAccelerometerViewModel.class);
        mAudiodetectionAndAccelerometerViewModel.init(surfaceView.getHolder());
        mAudiodetectionAndAccelerometerViewModel.getConnectedDeviceList().observe(this, new Observer<List<Device>>() {
            @Override
            public void onChanged(List<Device> devices) {
                deviceListAdapter.setDevices(devices);
            }
        });

        mAudiodetectionAndAccelerometerViewModel.getAlarmHistoryList().observe(this, new Observer<List<Device>>() {
            @Override
            public void onChanged(List<Device> devices) {
                alarmHistoryListAdapter.setAlarmHistoryList(devices);
            }
        });

        mAudiodetectionAndAccelerometerViewModel.getStartAlarmRecording().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean aBoolean) {
                if (aBoolean) {
                    if(permission.alarmRechtePruefen(AudiodetectionAndAccelerometerActivity.this, AudiodetectionAndAccelerometerActivity.this)){
                        Toast.makeText(AudiodetectionAndAccelerometerActivity.this, "Start Alarm Recording", Toast.LENGTH_LONG).show();
                        mAudiodetectionAndAccelerometerViewModel.startAlarmRecording();
                        //todo AlarmHandling einfügen
                    } else {
                        permission.alarmRechteAnfordern(AudiodetectionAndAccelerometerActivity.this);
                        if(permission.alarmRechtePruefen(AudiodetectionAndAccelerometerActivity.this, AudiodetectionAndAccelerometerActivity.this)){
                            Toast.makeText(AudiodetectionAndAccelerometerActivity.this, "Start Alarm Recording", Toast.LENGTH_LONG).show();
                            mAudiodetectionAndAccelerometerViewModel.startAlarmRecording();
                            //todo Alarmhandling einfügen
                        } else {
                            Toast.makeText(AudiodetectionAndAccelerometerActivity.this, "Stop Alarm Recording", Toast.LENGTH_LONG).show();
                            mAudiodetectionAndAccelerometerViewModel.stopAlarmRecording(AudiodetectionAndAccelerometerActivity.this);
                        }
                    }
                }
                else {
                    Toast.makeText(AudiodetectionAndAccelerometerActivity.this, "Stop Alarm Recording", Toast.LENGTH_LONG).show();
                    mAudiodetectionAndAccelerometerViewModel.stopAlarmRecording(AudiodetectionAndAccelerometerActivity.this);
                }
            }
        });

        mAudiodetectionAndAccelerometerViewModel.getAudiodetectionAndAccelerometerAlarmDetected().observe(this, new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean aBoolean) {
                if (aBoolean) {
                    Toast.makeText(AudiodetectionAndAccelerometerActivity.this, "onChangedTrue", Toast.LENGTH_LONG).show();
                    if (mAudiodetectionAndAccelerometerViewModel.getAccelerometerAlarmDetected() && !mAudiodetectionAndAccelerometerViewModel.getMicrofonAlarmDetected()) {
                        mAudiodetectionAndAccelerometerViewModel.updateDevice(mAudiodetectionAndAccelerometerViewModel.getLocalDeviceUUID(), mAudiodetectionAndAccelerometerViewModel.getSystemTimeStamp(), true, "Accelerometer", mAudiodetectionAndAccelerometerViewModel.getAccelerometerSensorMeanValue());
                    }
                    else if (mAudiodetectionAndAccelerometerViewModel.getMicrofonAlarmDetected() && !mAudiodetectionAndAccelerometerViewModel.getAccelerometerAlarmDetected()) {
                        mAudiodetectionAndAccelerometerViewModel.updateDevice(mAudiodetectionAndAccelerometerViewModel.getLocalDeviceUUID(), mAudiodetectionAndAccelerometerViewModel.getSystemTimeStamp(), true, "Audio", mAudiodetectionAndAccelerometerViewModel.getAmplitudeInDB());
                    }
                    else if (mAudiodetectionAndAccelerometerViewModel.getMicrofonAlarmDetected() && mAudiodetectionAndAccelerometerViewModel.getAccelerometerAlarmDetected()) {
                        mAudiodetectionAndAccelerometerViewModel.updateDevice(mAudiodetectionAndAccelerometerViewModel.getLocalDeviceUUID(), mAudiodetectionAndAccelerometerViewModel.getSystemTimeStamp(), true, "AudioAndAccelerometer", mAudiodetectionAndAccelerometerViewModel.getAmplitudeInDB());
                    }
                }
                else {
                    Toast.makeText(AudiodetectionAndAccelerometerActivity.this, "onChangedFalse", Toast.LENGTH_LONG).show();
                    mAudiodetectionAndAccelerometerViewModel.updateDevice(mAudiodetectionAndAccelerometerViewModel.getLocalDeviceUUID(), mAudiodetectionAndAccelerometerViewModel.getSystemTimeStamp(), false, "AudioAndAccelerometer", 0.0f);
                }

            }
        });

        accelerometerManager = (SensorManager) getSystemService(SENSOR_SERVICE);
        if (accelerometerManager.getSensorList(Sensor.TYPE_GYROSCOPE).size() == 0) {
            accelerometerSensor = null;
            Toast.makeText(AudiodetectionAndAccelerometerActivity.this, "No accelerometer sensor available", Toast.LENGTH_LONG).show();
        }
        else {
            accelerometerSensor = accelerometerManager.getSensorList(Sensor.TYPE_GYROSCOPE).get(0);
        }
    }

    @Override
    public void onSensorChanged(SensorEvent event) {
        StringBuilder sb = new StringBuilder();
        sb.append("x=")
                .append(event.values[0])
                .append("\ny=")
                .append(event.values[1])
                .append("\nz=")
                .append(event.values[2]);
        audiodetectionAndAccelerometerStatusMessage.setText(sb.toString());
        mAudiodetectionAndAccelerometerViewModel.addValueToGesamtBE(event.values[0] + event.values[1] + event.values[2]);
        mAudiodetectionAndAccelerometerViewModel.meanValueCalculation();
        mAudiodetectionAndAccelerometerViewModel.calibrateAccelerometerSensor();
        mAudiodetectionAndAccelerometerViewModel.checkAlarmCondition();
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {

    }

    @Override
    protected void onResume() {
        super.onResume();
        mAudiodetectionAndAccelerometerViewModel.startMicrofonSensor();
        if (accelerometerSensor != null) {
            if (accelerometerManager.registerListener(this, accelerometerSensor, SensorManager.SENSOR_DELAY_GAME)) {
                Toast.makeText(AudiodetectionAndAccelerometerActivity.this, "We registered to the sensor", Toast.LENGTH_LONG).show();
            } else {
                Toast.makeText(AudiodetectionAndAccelerometerActivity.this, "Registration did not work", Toast.LENGTH_LONG).show();
            }
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (isFinishing()){
            mAudiodetectionAndAccelerometerViewModel.recalibrationMicrofonSensor();
        }
        mAudiodetectionAndAccelerometerViewModel.stopMicrofonSensor();
        if (accelerometerSensor != null) {
            accelerometerManager.unregisterListener(this, accelerometerSensor);
        }
    }
}