package com.example.greenwatch.viewmodels;

import android.content.Context;
import android.view.SurfaceHolder;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModel;

import com.example.greenwatch.alarmrecorder.AlarmRecorder;
import com.example.greenwatch.models.Device;
import com.example.greenwatch.repositories.DeviceRepository;
import com.example.greenwatch.sensors.AccelerometerSensor;
import com.example.greenwatch.sensors.MicrofonSensor;

import java.util.List;

public class AudiodetectionAndAccelerometerViewModel extends ViewModel implements ViewModelInterface {
    private MutableLiveData<List<Device>> mDeviceList;
    private MutableLiveData<Boolean> mAccelerometerAlarmDetected;
    private MutableLiveData<Boolean> mMicrofonAlarmDetected;
    private MutableLiveData<Boolean> mAudiodetectionAndAccelerometerAlarmDetected = new MutableLiveData<>();
    private MutableLiveData<List<Device>> mAlarmHistoryList;
    private MutableLiveData<Boolean> mStartAlarmRecording;
    private AccelerometerSensor mAccelerometerSensor;
    private MicrofonSensor mMicrofonSensor;
    private AlarmRecorder mAlarmRecorder;
    private DeviceRepository mDeviceRepository;
    private boolean microfonAlarmDetected;
    private boolean accelerometerAlarmDetected;
    private boolean microfondetectionAndAccelerometerAlarmDetected;

    public void init() {

    }

    public void addValueToGesamtBE(float newValue) {
        mAccelerometerSensor.addValueToGesamtBE(newValue);
    }

    public void meanValueCalculation() {
        mAccelerometerSensor.meanValueCalculation();
    }

    public void calibrateAccelerometerSensor() {
        mAccelerometerSensor.calibrateAccelerometerSensor();
    }

    public void checkAlarmCondition() {
        mAccelerometerSensor.checkAlarmCondition();
    }

    public float getAccelerometerSensorMeanValue() {
        return mAccelerometerSensor.getAccelerometerSensorMeanValue();
    }

    public LiveData<Boolean> getAudiodetectionAndAccelerometerAlarmDetected() {
        setMutableLiveDataMicrofondetectionAndAccelerometerAlarmDetected();
        return mAudiodetectionAndAccelerometerAlarmDetected;
    }

    public void startMicrofonSensor() {
        mMicrofonSensor.start();
    }

    public void stopMicrofonSensor() {
        mMicrofonSensor.stop();
    }

    public void recalibrationMicrofonSensor() {
        mMicrofonSensor.doRecalibration();
    }

    public float getAmplitudeInDB() {
        return mMicrofonSensor.getAmplitudeInDB();
    }

    private void setMutableLiveDataMicrofondetectionAndAccelerometerAlarmDetected() {
        mAudiodetectionAndAccelerometerAlarmDetected.setValue(microfondetectionAndAccelerometerAlarmDetected);
    }

    private void setAccelerometerAlarmDetected(boolean accelerometerAlarmDetected) {
        this.accelerometerAlarmDetected = accelerometerAlarmDetected;
    }

    public boolean getAccelerometerAlarmDetected() {
        return accelerometerAlarmDetected;
    }

    private void setMicrofonAlarmDetected(boolean microfonAlarmDetected) {
        this.microfonAlarmDetected = microfonAlarmDetected;
    }

    public boolean getMicrofonAlarmDetected() {
        return microfonAlarmDetected;
    }

    Observer<Boolean> observer = new Observer<Boolean>() {
        @Override
        public void onChanged(Boolean aBoolean) {
            setAccelerometerAlarmDetected(mAccelerometerAlarmDetected.getValue());
            setMicrofonAlarmDetected(mMicrofonAlarmDetected.getValue());
            if (microfonAlarmDetected || accelerometerAlarmDetected) {
                microfondetectionAndAccelerometerAlarmDetected = true;
                setMutableLiveDataMicrofondetectionAndAccelerometerAlarmDetected();

            }
            else if (!microfonAlarmDetected && !accelerometerAlarmDetected) {
                microfondetectionAndAccelerometerAlarmDetected = false;
                setMutableLiveDataMicrofondetectionAndAccelerometerAlarmDetected();
            }
        }
    };

    private void registerAlarmObserver() {
        mMicrofonAlarmDetected.observeForever(observer);
        mAccelerometerAlarmDetected.observeForever(observer);
    }

    @Override
    protected void onCleared() {
        super.onCleared();
        mMicrofonAlarmDetected.removeObserver(observer);
        mAccelerometerAlarmDetected.removeObserver(observer);
    }

    @Override
    public void init(SurfaceHolder previewHolder) {
        if (mDeviceRepository == null) {
            mDeviceRepository = DeviceRepository.getInstance();
        }
        if (mAccelerometerSensor == null) {
            mAccelerometerSensor = AccelerometerSensor.getInstance();
        }
        if (mMicrofonSensor == null) {
            mMicrofonSensor = MicrofonSensor.getInstance();
        }
        if (mAlarmRecorder == null) {
            mAlarmRecorder = AlarmRecorder.getInstance();
        }
        if (mDeviceList == null) {
            mDeviceList = mDeviceRepository.getConnectedDeviceList();
        }
        if (mAlarmHistoryList == null) {
            mAlarmHistoryList = mDeviceRepository.getAlarmHistoryDeviceList();
        }
        if (mStartAlarmRecording == null) {
            mStartAlarmRecording = mDeviceRepository.getStartAlarmRecording();
        }
        if (mAccelerometerAlarmDetected == null) {
            mAccelerometerAlarmDetected = mAccelerometerSensor.getAccelerometerAlarmDetected();
        }
        if (mMicrofonAlarmDetected == null) {
            mMicrofonAlarmDetected = mMicrofonSensor.getAccelerometerAlarmDetected();
        }
        microfondetectionAndAccelerometerAlarmDetected = false;
        accelerometerAlarmDetected = false;
        microfonAlarmDetected = false;
        registerAlarmObserver();
        setAlarmRecordingPreviewHolder(previewHolder);
    }

    @Override
    public LiveData<List<Device>> getConnectedDeviceList() {
        return mDeviceList;
    }

    @Override
    public LiveData<List<Device>> getAlarmHistoryList() {
        return mAlarmHistoryList;
    }

    @Override
    public LiveData<Boolean> getStartAlarmRecording() {
        return mStartAlarmRecording;
    }

    @Override
    public void updateDevice(String deviceID, String timeStamp, boolean sensorStatus, String sensorType, float sensorMassage) {
        mDeviceRepository.updateDevice(deviceID, timeStamp, sensorStatus, sensorType, sensorMassage);
    }

    @Override
    public void setTimeStamp(String deviceID, String timeStamp) {
        mDeviceRepository.setTimeStamp(deviceID, timeStamp);
    }

    @Override
    public String getTimeStamp(String deviceID) {
        return mDeviceRepository.getTimeStamp(deviceID);
    }

    @Override
    public void setDeviceID(String deviceID, String newDeviceID) {
        mDeviceRepository.setDeviceID(deviceID, newDeviceID);
    }

    @Override
    public String getDeviceID(String deviceID) {
        return mDeviceRepository.getDeviceID(deviceID);
    }

    @Override
    public void setSensorStatus(String deviceID, boolean sensorStatus) {
        mDeviceRepository.setSensorStatus(deviceID, sensorStatus);
    }

    @Override
    public boolean getSensorStatus(String deviceID) {
        return mDeviceRepository.getSensorStatus(deviceID);
    }

    @Override
    public void setSensorType(String deviceID, String sensorType) {
        mDeviceRepository.setSensorType(deviceID, sensorType);
    }

    @Override
    public String getSensorType(String deviceID) {
        return mDeviceRepository.getSensorType(deviceID);
    }

    @Override
    public void setSensorMassage(String deviceID, float sensorMessage) {
        mDeviceRepository.setSensorMassage(deviceID, sensorMessage);
    }

    @Override
    public float getSensorMassage(String deviceID) {
        return mDeviceRepository.getSensorMassage(deviceID);
    }

    @Override
    public String getLocalDeviceUUID() {
        return mDeviceRepository.getLocalDeviceUUID();
    }

    @Override
    public String getSystemTimeStamp() {
        return mDeviceRepository.getSystemTimeStamp();
    }

    @Override
    public void startAlarmRecording() {
        mAlarmRecorder.startRecording();
    }

    @Override
    public void stopAlarmRecording(Context context) {
        mAlarmRecorder.stopRecording(context);
    }

    @Override
    public void setAlarmRecordingPreviewHolder(SurfaceHolder previewHolder) {
        mAlarmRecorder.setPreviewHolder(previewHolder);
    }
}
